/**
 *  Copyright (c) 2014,
 *  BTC Embedded Systems AG, Oldenburg, Germany
 *  All rights reserved
 *  ---------------------------------------------------------
 *  $Author: $
 *  $Date:  $
 *  $Revision: $
 **/
package com.btc.ep.base.dal.impl;

import java.util.Collection;

import com.btc.ep.base.bl.ModelElement;
import com.btc.ep.base.bl.impl.ModelElementImpl;

/**
 * This interface provide the base functions for all domain specific repository's.
 * The Repository's are used by the Business Layer for access to the DataBase.
 * 
 * @param <T> Type of ModelElement
 */
public interface BaseRepository<T extends ModelElement> {

    /**
     * Find the repository aggregate by Uid
     * 
     * @param uid
     *            The Uid of the ModelElement to load.
     *            Attention! It must be an uid of the type of aggregate.
     * @return The ModelElement with the given uid. If no element found it return NULL.
     */
    public T find(String uid);

    /**
     * Find all aggregates of the repository
     *
     * @param <E> Type of an Entity (ModelElementImpl)
     * @param clazz The Implementation class of the repository aggregate.
     * @return All ModelElements of the repository aggregate Type. If no element found it return NULL.
     */
    public <E extends ModelElementImpl> Collection<E> findAll(Class<E> clazz);

    /**
     * Adds a ModelElement to the data repository if it not exist yet (add also all new Elements in the object graph).
     * <p><b>ATTATTION:</b> The update detach the ModelElement!</p>
     *
     * @param dmo The dmo to be updated, must not be null.
     * @return The new attached dmo. Now all changes on the DMO will be observe automatic by the Provider. That means,
     *         that all modifications on the DMO will be stored in the Database at next save.
     */
    public T update(T dmo);

    /**
     * Adds every ModelElements of the collection to the data repository if it not exist yet (add also all new Elements
     * in the object graph).
     *
     * @param dmos
     *            Collection that contain all modelElements to be updated.
     *            The collection must not be null and all elements of the collection to be updated must not be null.
     * @return Collection with the new attached dmos. Now all changes on the DMO will be observe automatic by the
     *         Provider. That means,
     *         that all modifications on the DMO will be stored in the Database at next save.
     */
    public Collection<T> update(Collection<T> dmos);

    /**
     * Deletes a modelElement.
     * This function deleting the element itself and all database references.
     *
     * @param dmo The modelElement to be deleted.
     */
    public void remove(T dmo);

    /**
     * Deletes multiple modelElements.
     *
     * @param dmos Collection that contain all modelElements to be deleted.
     */
    public void remove(Collection<T> dmos);

    /**
     * Deletes multiple modelElementImpls.
     *
     * @param dmos Collection that contain all modelElements to be deleted.
     */
    public void removeImpl(Collection<? extends ModelElementImpl> dmos);

    /**
     * Save the changes in the persistenceContext to the DataBase file.
     * 
     * @return success.
     */
    public boolean save();

    /**
     * Revert all not saved modifications in the persitenceContext.
     * This method must be called, if modifications can't be completed and the DMO's have a inconsistent state.
     *
     *<p><b>ATTATTION:</b> The revert detach <b>ALL</b> loaded ModelElements!</p>
     *
     * @return success.
     */
    public boolean revert();

    /**
     * Clear the Database. 
     * After execution all ModelElements deleted in the persistenceContext and all DMO's in BL are detached.
     *
     * @return success.
     */
    public boolean clearDatabase();
}
